// This is the body of a uintptr_t array definition. Fill in your own addresses.
0x116bb312c, // example values (can be >32 bit)
0x38d3c5ad,  // replace with your own
0x77c1e96d   // don't forget: no comma after the last one
